document.addEventListener('deviceready', initStore);

var productInfo = {};

var CordovaStore = {};
var callbackProducts = [];
var firstPurchaseCallbackProducts = [];
var callbackFunc = null;
var firstPurchaseCallback = null;
var afterAllProductsFunc = null;

CordovaStore.isAvailable = function() {
    return false;
}

CordovaStore.setHasProductCallback = function(callback, fpCallback, afterAllProducts) {
    callbackFunc = callback;
    firstPurchaseCallback = fpCallback;
    afterAllProductsFunc = afterAllProducts;

    while (callbackProducts.length > 0) {
        var prod = callbackProducts.pop();
        callback(prod);
    }

    var hadAny = firstPurchaseCallbackProducts.length > 0;
    if (hadAny) {
        firstPurchaseCallback(firstPurchaseCallbackProducts);
    }
    if (hadAny) {
        afterAllProducts();
    }
}

function initStore() {
    if (!window.store) {
        console.log('Store not available');
        return;
    }

    store.register({
        id: 'remove_ads',
        alias: 'remove_ads',
        type: store.NON_CONSUMABLE
    });
    store.register({
        id: 'premium_upgrade',
        alias: 'premium_upgrade',
        type: store.NON_CONSUMABLE
    });
    store.register({
        id: 'premium',
        alias: 'premium',
        type: store.NON_CONSUMABLE
    });

    store.error(function(error) {
        console.log('ERROR ' + error.code + ': ' + error.message);
    });

    store.when("product").updated(function(product) {
        productInfo[product.id] = product;

        if (product.owned) {
            
            if (callbackFunc == null) {
                callbackProducts.push(product.id);
            } else {
                callbackFunc(product.id);
                afterAllProductsFunc();
            }
        }
    });

    //test consume products
    // store.when("product").updated(product => {
    //     if(product.owned) {
    //         var transaction = product.transaction;
    //         store.inappbilling.consumePurchase(
    //             function() { // success
    //                 alert("consume success");
    //             },
    //             function(err, code) { // error
    //                 alert("consume error " + err)
    //             },
    //             product.id,
    //             transaction.id
    //         );
    //     }
    // });

    //Show message on first purchase of anything
    (function(ack) {
        store.inappbilling.acknowledgePurchase = function (success, fail, productId, transactionId, developerPayload) {
            ack.call(this, success, fail, productId, transactionId, developerPayload);
            
            if (firstPurchaseCallback == null) {
                firstPurchaseCallbackProducts.push(productId);
            } else {
                firstPurchaseCallback([productId]);
            }
        };
    }(store.inappbilling.acknowledgePurchase));

    store.when("product").approved(function(product) {
        productInfo[product.id] = product;
        
        product.finish();
    });

    store.refresh();

    CordovaStore.isAvailable = function() {
        return true;
    }

    CordovaStore.getProductInfo = function(product_id) {
        return productInfo[product_id];
    }

    CordovaStore.purchase = function(product_id) {
        store.order(product_id);
        return true;
    }
}